@@ -39,6 +39,8 @@ let prepare_target llmod =
 let new_module () =
   let m = Llvm.create_module context module_name in
   let () = prepare_target m in
+  let () = Llvm_debuginfo.set_is_new_dbg_info_format m true in
+  insist (Llvm_debuginfo.is_new_dbg_info_format m);
   m
 
 let test_get_module () =
@@ -110,7 +112,18 @@ let test_get_function m dibuilder file_di m_di =
   stdout_metadata int_ty_di;
   (* CHECK: [[INT32_PTR:<0x[0-9a-f]*>]] = !DIBasicType(name: "int", size: 32, encoding: DW_ATE_signed)
   *)
-  let param_types = [| null_metadata; int_ty_di |] in
+  let int_ptr_ty_di =
+    Llvm_debuginfo.dibuild_create_pointer_type dibuilder
+      ~pointee_ty:int_ty_di
+      ~size_in_bits:32
+      ~align_in_bits:32
+      ~address_space:0
+      ~name:"ptrint"
+  in
+  stdout_metadata int_ptr_ty_di;
+  (* CHECK: [[PTRINT32_PTR:<0x[0-9a-f]*>]] = !DIDerivedType(tag: DW_TAG_pointer_type, name: "ptrint", baseType: [[INT32_PTR]], size: 32, align: 32, dwarfAddressSpace: 0)
+  *)
+  let param_types = [| null_metadata; int_ty_di; int_ptr_ty_di |] in
   let fty_di =
     Llvm_debuginfo.dibuild_create_subroutine_type dibuilder ~file:file_di
       ~param_types flags_zero
@@ -124,7 +137,7 @@ let test_get_function m dibuilder file_di m_di =
     Llvm_debuginfo.dibuild_get_or_create_type_array dibuilder ~data:param_types
   in
   stdout_metadata fty_di_args;
-  (* CHECK: [[FARGS_PTR:<0x[0-9a-f]*>]] = !{null, [[INT32_PTR]]}
+  (* CHECK: [[FARGS_PTR:<0x[0-9a-f]*>]] = !{null, [[INT32_PTR]], [[PTRINT32_PTR]]}
   *)
   stdout_metadata fty_di;
   (* CHECK: [[SBRTNTY_PTR:<0x[0-9a-f]*>]] = !DISubroutineType(types: [[FARGS_PTR]])
@@ -132,7 +145,8 @@ let test_get_function m dibuilder file_di m_di =
   (* Let's create the LLVM-IR function now. *)
   let name = "tfun" in
   let fty =
-    Llvm.function_type (Llvm.void_type context) [| Llvm.i32_type context |]
+    Llvm.function_type (Llvm.void_type context)
+      [| Llvm.i32_type context; Llvm.pointer_type context |]
   in
   let f = Llvm.define_function name fty m in
   let f_di =
@@ -142,7 +156,7 @@ let test_get_function m dibuilder file_di m_di =
       ~flags:flags_zero ~is_optimized:false
   in
   stdout_metadata f_di;
-  (* CHECK: [[SBPRG_PTR:<0x[0-9a-f]*>]] = distinct !DISubprogram(name: "tfun", linkageName: "tfun", scope: [[MODULE_PTR]], file: [[FILE_PTR]], line: 10, type: [[SBRTNTY_PTR]], scopeLine: 10, spFlags: DISPFlagDefinition, unit: [[CMPUNIT_PTR]], retainedNodes: {{<0x[0-9a-f]*>}})
+  (* CHECK: [[SBPRG_PTR:<0x[0-9a-f]*>]] = distinct !DISubprogram(name: "tfun", linkageName: "tfun", scope: [[MODULE_PTR]], file: [[FILE_PTR]], line: 10, type: [[SBRTNTY_PTR]], scopeLine: 10, spFlags: DISPFlagDefinition, unit: [[CMPUNIT_PTR]])
   *)
   Llvm_debuginfo.set_subprogram f f_di;
   ( match Llvm_debuginfo.get_subprogram f with
@@ -152,28 +166,25 @@ let test_get_function m dibuilder file_di m_di =
     ( Llvm_debuginfo.get_metadata_kind f_di
     = Llvm_debuginfo.MetadataKind.DISubprogramMetadataKind );
   insist (Llvm_debuginfo.di_subprogram_get_line f_di = 10);
-  (f, f_di)
+  (fty, f, f_di)
 
-let test_bbinstr f f_di file_di dibuilder =
+let test_bbinstr fty f f_di file_di dibuilder =
   group "basic_block and instructions tests";
   (* Create this pattern:
    *   if (arg0 != 0) {
-   *      foo(arg0);
+   *      foo(arg0, arg1);
    *   }
    *   return;
    *)
   let arg0 = (Llvm.params f).(0) in
+  let arg1 = (Llvm.params f).(1) in
   let builder = Llvm.builder_at_end context (Llvm.entry_block f) in
   let zero = Llvm.const_int (Llvm.i32_type context) 0 in
   let cmpi = Llvm.build_icmp Llvm.Icmp.Ne zero arg0 "cmpi" builder in
   let truebb = Llvm.append_block context "truebb" f in
   let falsebb = Llvm.append_block context "falsebb" f in
   let _ = Llvm.build_cond_br cmpi truebb falsebb builder in
-  let foodecl =
-    Llvm.declare_function "foo"
-      (Llvm.element_type (Llvm.type_of f))
-      (Llvm.global_parent f)
-  in
+  let foodecl = Llvm.declare_function "foo" fty (Llvm.global_parent f) in
   let _ =
     Llvm.position_at_end truebb builder;
     let scope =
@@ -187,7 +198,7 @@ let test_bbinstr f f_di file_di dibuilder =
       | Some file_of_f_di', Some file_of_scope' ->
           file_of_f_di' = file_di && file_of_scope' = file_di
       | _ -> false );
-    let foocall = Llvm.build_call foodecl [| arg0 |] "" builder in
+    let foocall = Llvm.build_call fty foodecl [| arg0; arg1 |] "" builder in
     let foocall_loc =
       Llvm_debuginfo.dibuild_create_debug_location context ~line:10 ~column:12
         ~scope
@@ -260,6 +271,52 @@ let test_global_variable_expression dibuilder f_di m_di =
    *)
   ()
 
+let test_variables f dibuilder file_di fun_di =
+  let entry_term = Option.get @@ (Llvm.block_terminator (Llvm.entry_block f)) in
+  group "Local and parameter variable tests";
+  let ty = int_ty_di 64 dibuilder in
+  stdout_metadata ty;
+  (* CHECK: [[INT64TY_PTR:<0x[0-9a-f]*>]] = !DIBasicType(name: "int", size: 64, encoding: DW_ATE_signed)
+  *)
+  let auto_var =
+    Llvm_debuginfo.dibuild_create_auto_variable dibuilder ~scope:fun_di
+      ~name:"my_local" ~file:file_di ~line:10 ~ty
+      ~always_preserve:false flags_zero ~align_in_bits:0
+  in
+  stdout_metadata auto_var;
+  (* CHECK: [[LOCAL_VAR_PTR:<0x[0-9a-f]*>]] = !DILocalVariable(name: "my_local", scope: <{{0x[0-9a-f]*}}>, file: <{{0x[0-9a-f]*}}>, line: 10, type: [[INT64TY_PTR]])
+  *)
+  let builder = Llvm.builder_before context entry_term in
+  let all = Llvm.build_alloca (Llvm.i64_type context)  "my_alloca" builder in
+  let scope =
+    Llvm_debuginfo.dibuild_create_lexical_block dibuilder ~scope:fun_di
+      ~file:file_di ~line:9 ~column:4
+  in
+  let location =
+    Llvm_debuginfo.dibuild_create_debug_location
+    context ~line:10 ~column:12 ~scope
+  in
+  let vdi = Llvm_debuginfo.dibuild_insert_declare_before dibuilder ~storage:all
+    ~var_info:auto_var ~expr:(Llvm_debuginfo.dibuild_expression dibuilder [||])
+    ~location ~instr:entry_term
+  in
+  let () = Printf.printf "%s\n" (Llvm.string_of_lldbgrecord vdi) in
+  (* CHECK: dbg_declare(ptr %my_alloca, ![[#]], !DIExpression(), ![[#]])
+  *)
+  let arg1 = (Llvm.params f).(1) in
+  let arg_var = Llvm_debuginfo.dibuild_create_parameter_variable dibuilder ~scope:fun_di
+    ~name:"my_arg" ~argno:1 ~file:file_di ~line:10 ~ty
+    ~always_preserve:false flags_zero
+  in
+  let argdi = Llvm_debuginfo.dibuild_insert_declare_before dibuilder ~storage:arg1
+    ~var_info:arg_var ~expr:(Llvm_debuginfo.dibuild_expression dibuilder [||])
+    ~location ~instr:entry_term
+  in
+  let () = Printf.printf "%s\n" (Llvm.string_of_lldbgrecord argdi) in
+  (* CHECK: dbg_declare(ptr %1, ![[#]], !DIExpression(), ![[#]])
+  *)
+  ()
+
 let test_types dibuilder file_di m_di =
   group "type tests";
   let namespace_di =
@@ -400,9 +457,10 @@ let test_types dibuilder file_di m_di =
 
 let () =
   let m, dibuilder, file_di, m_di = test_get_module () in
-  let f, fun_di = test_get_function m dibuilder file_di m_di in
-  let () = test_bbinstr f fun_di file_di dibuilder in
+  let fty, f, fun_di = test_get_function m dibuilder file_di m_di in
+  let () = test_bbinstr fty f fun_di file_di dibuilder in
   let () = test_global_variable_expression dibuilder file_di m_di in
+  let () = test_variables f dibuilder file_di fun_di in
   let () = test_types dibuilder file_di m_di in
   Llvm_debuginfo.dibuild_finalize dibuilder;
   ( match Llvm_analysis.verify_module m with
